package debugger;

import java.io.*;
import java.net.*;
import java.util.*;
import debugger.object.*;

public class Debugger implements Constants {
  public final IO io;

  public Debugger(InetAddress kernelAddress, int kernelPort) {
    io = new TCPIO(kernelAddress, kernelPort);
  }
  public Debugger(File logFile, File actLogFile, File agentLogFile) {
    io = new LogIO(logFile, actLogFile);
    if(agentLogFile != null) AGENT_VIEWER.setLogfile(agentLogFile);
  }

  public void check() {
    io.connect();
    new Thread(io).start();
    preCheck();
    VIEWER.setVisible(true);
    if(AGENT_VIEWER.getFile() != null)AGENT_VIEWER.setVisible(true);
    while (true) {
      while(!io.hasUpdateData(WORLD.time() + 1))
        try { Thread.sleep(50);}
        catch (InterruptedException e) { /*assert false;*/ }
      AGENT_VIEWER.progress();
      STATUS_VIEWER.progress();
      WORLD.parseCommands();
      if (VIEWER.doAnimation())
	     VIEWER.animate();
      WORLD.progress();
      if (!VIEWER.doAnimation())
	     VIEWER.repaintWithWaiting();

      if (WORLD.time() == 2)
      printNumOfObjects();
      if (WORLD.time() == 300)
      printObjcectList(WORLD.movingObjectList, new PropertyFunction() {
      public String name() { return "dead buriedness"; }
      public int eval(Object obj) { return (((Humanoid) obj).hp() == 0) ? ((Humanoid) obj).buriedness() : -1; }});
    }
  }

  private void printNumOfObjects() {
    pr("width: " + ((WORLD.maxX() - WORLD.minX()) / 1000) + " [m]");
    pr("height:" + ((WORLD.maxY() - WORLD.minY()) / 1000) + " [m]");
    pr("Object:" + WORLD.idObjMap.size());
    pr("Building:" + WORLD.buildingList.size());
    printObjcectList(WORLD.buildingList, new PropertyFunction() {
	public String name() { return "brokenness"; }
	public int eval(Object obj) { return ((Building) obj).brokenness(); }});
    printObjcectList(WORLD.buildingList, new PropertyFunction() {
	public String name() { return "type"; }
	public int eval(Object obj) { return ((RescueObject) obj).type(); }});
    pr("Road:" + WORLD.roadList.size());
    printObjcectList(WORLD.roadList, new PropertyFunction() {
	public String name() { return "aliveLinesOnEachDirection"; }
	public int eval(Object obj) { return ((Road) obj).aliveLinesToHead(); }});
    pr("Node:" + WORLD.nodeList.size());
    pr("MovingObj:" + WORLD.movingObjectList.size());
    printObjcectList(WORLD.movingObjectList, new PropertyFunction() {
	public String name() { return "buriedness"; }
	public int eval(Object obj) { return ((Humanoid) obj).buriedness(); }});
    pr();
  }

  private void printObjcectList(List list, PropertyFunction func) {
    HashMap map = new HashMap();
    Iterator it = list.iterator();
    while (it.hasNext()) {
      Object key = new Integer(func.eval(it.next()));
      Object num = map.get(key);
      num = new Integer((num == null) ? 1 : ((Integer) num).intValue() + 1);
      map.put(key, num);
    }

    pr("  " +  func.name());
    List tmp = new ArrayList(map.keySet());
    Collections.sort(tmp);
    it = new ArrayList(tmp).iterator();
    while (it.hasNext()) {
      Integer key = (Integer) it.next();
      pr("  " + key + "\t:" + map.get(key));
    }
  }

  // -------------------------------------------------------------------- check
  private void preCheck() {
    pr("Object:   " + WORLD.idObjMap.size());
    pr("Building: " + WORLD.buildingList.size());
    pr("Road:     " + WORLD.roadList.size());
    pr("Node:     " + WORLD.nodeList.size());
    pr("MovingObj:" + WORLD.movingObjectList.size());
    pr();
    pr("width:  " + ((WORLD.maxX() - WORLD.minX()) / 1000) + " [m]");
    pr("height: " + ((WORLD.maxY() - WORLD.minY()) / 1000) + " [m]");
    pr();

    pr("Is the GIS Map a connected graph?");
    if (isTheGISMapAConnectedGraph())
      pr("  ... yes");

    pr();
  }

  // ---------------------------------------- Is the GIS Map a connected graph?
  private boolean isTheGISMapAConnectedGraph() {
    boolean result = true;
    Set visited = new HashSet();
    //assert !WORLD.buildingList.isEmpty();
    MotionlessObject org = (MotionlessObject) WORLD.buildingList.get(0);
    checkTheGISMapIsAConnectedGraph(org, visited);
    Iterator it = WORLD.idObjMap.values().iterator();
    while (it.hasNext()) {
      Object obj = it.next();
      if (!visited.contains(obj)  &&  obj instanceof MotionlessObject) {
	result = false;
	pr(errMsg("The GIS map is not a connected graph.  A graph containing " + obj + " does not connect other graph."));
	checkTheGISMapIsAConnectedGraph((MotionlessObject) obj, visited);
      }
    }
    return result;
  }

  private void checkTheGISMapIsAConnectedGraph(MotionlessObject org, Set visited) {
    if (visited.contains(org))
      return;
    visited.add(org);
    if (org instanceof Road) {
      Road road = (Road) org;
      checkTheGISMapIsAConnectedGraph(road.head(), visited);
      checkTheGISMapIsAConnectedGraph(road.tail(), visited);
    } else if (org instanceof Node) {
      Iterator it = ((Node) org).edges().iterator();
      while (it.hasNext())
	checkTheGISMapIsAConnectedGraph((MotionlessObject) it.next(), visited);
    } else if (org instanceof Building) {
      Set entrances = ((Building) org).entrances();
//      assert entrances.size() == 1 : errMsg("There is a building which have not just one entrance: " + org);
      checkTheGISMapIsAConnectedGraph((MotionlessObject) entrances.iterator().next(), visited);
    }
  }

  private void pr() { pr(""); }
  private void pr(String str) { System.out.println(str); }
  private String errMsg(String msg) { return "    error: " + msg; }

  ///////////
  private interface PropertyFunction {
    String name();
    int eval(Object obj);
  }
}
