package debugger;

import java.io.*;
import java.net.*;
import java.util.*;
import debugger.object.*;

public abstract class IO implements Runnable, Constants {
  protected abstract byte[] receive();
  protected abstract void send(int header, byte[] body);

  protected final ArrayList updateDataList   = new ArrayList();
  protected final ArrayList commandsDataList = new ArrayList();

  public IO() {
    commandsDataList.add(null);  // Dummy, because KS_COMMANDS is not sent by kernel at time 0
  }

  public void connect() {
    System.out.print("connecting ... ");
    send(VK_CONNECT, new byte[] {0, 0, 0, 0});  // version must be 0
    byte[] data = receive();
    System.out.println("done");

    System.out.print("initializing ... ");
//    storeData(updateDataList, data, 0);
    DataInputStream in = new DataInputStream(new ByteArrayInputStream(data));
    int size;
    try {
    int header = in.readInt();
    byte[] body;
    switch (header) {
      case KV_CONNECT_OK:
        size = in.readInt();
        System.out.print("initializing ... ");
        body = new byte[size];
        in.read(body);
        storeData(updateDataList, body, 0);
        break;
      case KV_CONNECT_ERROR:
        size = in.readInt();
//            System.out.println("Error connecting to kernel: "+in.readString());
        System.exit( -1);
        break;
      case UPDATE:
        size = in.readInt();
        int time = in.readInt();
        body = new byte[size];
        in.read(body);
        storeData(updateDataList, body, time);
        break;
      default:
        System.err.println("Unexpected reply from kernel: " + header);
        System.exit( -2);
        break;
    }
    } catch(Exception e){}
    WORLD.playback(0);
    WORLD.setWorldRange();
    System.out.println("done");

    send(VK_ACKNOWLEDGE, new byte[0]);
    System.out.println("start\n");
  }

  public void run() {
    int lastTime = WORLD.time();
    while (true) {
            byte[] data = receive();
            if (data == null)
                    return;
            DataInputStream dis = new DataInputStream(new ByteArrayInputStream(data));
            //			rescuecore.Handy.printBytes("Next block of data",dis);
            try {
              int header = dis.readInt();
              while (header != HEADER_NULL) {
                int size = dis.readInt();
                byte[] content = new byte[size];
                dis.read(content);
                switch (header) {
                  case COMMANDS:
                    storeData(commandsDataList, content, lastTime + 1);
                    break;
                  case UPDATE:
                    DataInputStream in = new DataInputStream(new ByteArrayInputStream(
                        content));
                    int time = in.readInt();
                    lastTime = time;
                    storeData(updateDataList, content, time);
                    break;
                  default:
                    assert false;
                }
                header = dis.readInt();
              }
            } catch(IOException e) {e.printStackTrace();}
    }
  }

  private void storeData(List list, byte[] data, int time) {
  synchronized (list) {
      if (list.size() > time)
	return;
      assert list.size() == time: "Some datas was skipped or lost.";
      list.add(data);
    }
  }

  public DataInputStream updateData(int time)   { return getData(time, updateDataList,  (time == 0) ? 0 : 1); }
  public DataInputStream commandsData(int time) { return getData(time, commandsDataList, 0); }
  private DataInputStream getData(int time, List list, int skip) {
    byte[] data;
    synchronized (list) { data = (byte[]) list.get(time); }
    DataInputStream dis = new DataInputStream(new ByteArrayInputStream(data));
    try { dis.skip(4 * skip); } catch (IOException e) { assert false; }
    return dis;
  }

  public boolean hasUpdateData(int time)   { synchronized (updateDataList)   { return time < updateDataList.size();   } }
  public boolean hasCommandsData(int time) { synchronized (commandsDataList) { return time < commandsDataList.size(); } }
}
