package debugger;

import java.io.*;
import java.net.*;

public class Main implements Constants{
  private static InetAddress m_kernelAddress;
  private static int         m_kernelPort;
  private static File        m_logFile = null;
  private static File        m_actLogFile = null;
  private static File        m_agentLogFile = null;
  private static String m_teamName = null;
  public static String teamName() { return m_teamName; }

  public static Debugger m_debugger;
  public static Debugger debugger() { return m_debugger; }

  public static void main(String[] args) {
    parseArgs(args);
    m_debugger = (m_logFile instanceof File)
      ? new Debugger(m_logFile, m_actLogFile, m_agentLogFile)
      : new Debugger(m_kernelAddress, m_kernelPort);
    try{
      Confing.readConfingFile();
    } catch(FileNotFoundException e) {
      try{
        Confing.writeConfingFile();
      } catch(IOException ioe) {}
    }
    m_debugger.check();
  }

  private static void parseArgs(String[] args) {
    try {
      final int UDP = 0x01;
      final int LOG = 0x02;
      int flag = 0x00;
      m_kernelAddress = InetAddress.getByName("localhost"); // .getLocalHost();
      m_kernelPort    = DEFAULT_KERNEL_PORT;
      int i = 0;
      while (i < args.length) {
	char   option = args[i ++].charAt(1);
	String val    = args[i ++];
	switch(option) {
	  default:   printUsage();
	  case 't':  m_teamName = val;  break;
	  case 'h':  flag |= UDP;  assert flag == UDP;  m_kernelAddress = InetAddress.getByName(val);  break;
	  case 'p':  flag |= UDP;  assert flag == UDP;  m_kernelPort    = Integer.parseInt(val);       break;
	  case 'l':  flag |= LOG;  assert flag == LOG;  m_logFile    = new File(val); break;
	  case 'a':  flag |= LOG;  assert flag == LOG;  m_actLogFile = new File(val); break;
          case 's':  flag |= LOG;  assert flag == LOG;  m_agentLogFile = new File(val); break;
	}
      }
    } catch (Exception e) {
      printUsage();
    } catch (AssertionError e) {
      printUsage();
    }
  }

  private static void printUsage() {
    String usage
      = "usage: java debugger.Main [-t <team>] [[-h <host>][-p <port>]|[-l <log>][-a <actlog>]]\n"
      + " <team>   team name\n"
      + " <host>   kernel host (default: localhost)\n"
      + " <port>   kernel port (default: " + DEFAULT_KERNEL_PORT + ")\n"
      + " <log>    log file\n"
      + " <actlog> action log file\n"
      + "          (This debugger saves \"" + ACTION_LOG_FILE_NAME + "\" as the action log file)\n";
    System.err.print(usage);
    System.exit(1);
  }
}
