package debugger;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import debugger.object.*;

public class Viewer extends JFrame implements Constants {
  private boolean m_doPlayBack  = false;
  public  boolean doPlayBack() { return m_doPlayBack; }
  private boolean m_doAnimation = VIEWER_DO_ANIMATION_AS_A_DEFAULT;
  public boolean doAnimation() { return m_doAnimation; }
  private int m_intervalEachCycle = VIEWER_DEFAULT_INTERVAL_EACH_CYCLE;

  private ArrayList clickedObjects = new ArrayList();

  private Image m_motionlessObjectsImage;
  private boolean m_wasChangedBackground;
  private boolean m_isWaiting = false;
  private int m_attendingId = -1;
  public final JSlider timeSlider = new JSlider(JSlider.HORIZONTAL, 0, 300, 0);
  public final JTextField attendingIdField = new JTextField();
  public final JTextField timeField = new JTextField();
  public final PaintField m_paintField = new PaintField(); ///add 19/05/02
  public final JScrollBar m_vScroll = new JScrollBar(JScrollBar.VERTICAL),
                          m_hScroll = new JScrollBar(JScrollBar.HORIZONTAL);


  public Viewer() {
    super("RescueViewer ver 0.3");
    addWindowListener(new ViewerWindowAdapter());
    setMenu();
    Container p0 = getContentPane();
    p0.setLayout(new BorderLayout());
    p0.add(controlPanel(), BorderLayout.NORTH);
    JPanel p1 = new JPanel(new BorderLayout());
    p1.add(statusPanel(),    BorderLayout.NORTH);
    p1.add(m_paintField, BorderLayout.CENTER);
    setScrollBar(p1);
    p0.add(p1, BorderLayout.CENTER);
    pack();
    setSize(VIEWER_WIDTH, VIEWER_HEIGHT);
  }

  private void setMenu() {
    JMenuBar rootMenuBar = new JMenuBar();
    JMenu viewMenu = new JMenu("view");
    final JMenuItem mi_agentLogView = new JMenuItem("AgentLog");
    final JMenuItem mi_statusView   = new JMenuItem("Status");
    final JMenuItem mi_zoomOut      = new JMenuItem("Zoom out");
    viewMenu.add(mi_agentLogView);  viewMenu.add(mi_statusView);
    viewMenu.add(mi_zoomOut);
    rootMenuBar.add(viewMenu);
    this.setJMenuBar(rootMenuBar);
    //// listener
    ActionListener actionlistener = new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();
        if(o == mi_agentLogView) AGENT_VIEWER.setVisible(true);
        else if(o == mi_statusView) STATUS_VIEWER.setVisible(true);
        else if(o == mi_zoomOut) m_paintField.setMagnification(1.0);
      }
    };
    mi_agentLogView.addActionListener(actionlistener);
    mi_statusView.addActionListener(actionlistener);
    mi_zoomOut.addActionListener(actionlistener);
  }

  private JToolBar controlPanel() {
    JToolBar panel = new JToolBar();
    panel.setPreferredSize(new Dimension(VIEWER_WIDTH, 30));
    panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
    JButton button = addStopButton(panel);
    addTimeField(panel, button);
    addAttendingIdField(panel);
    addIntervalEachCycleSlider(panel);
    addDoAnimateCheckBox(panel);
    return panel;
  }

  private JButton addStopButton(JComponent panel) {
    final ImageIcon stopImage = new ImageIcon("image/player_stop.gif");
    final ImageIcon playImage = new ImageIcon("image/Play24.gif");
    final ImageIcon forwardImage = new ImageIcon("image/Forward24.gif");
    final ImageIcon backImage = new ImageIcon("image/Back24.gif");
    final JButton stopButton = new JButton(stopImage);
    final JButton reverseButton = new JButton(forwardImage);
    stopButton.setPreferredSize(new Dimension(50, 20));
    stopButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        m_isWaiting = ! m_isWaiting;
        if (m_isWaiting) {
          stopButton.setIcon(playImage);
          WORLD.playback(WORLD.time());
          repaint();
          m_doRedo = true;
          return;
        }
        stopButton.setIcon(stopImage);
      }});
    reverseButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        m_doPlayBack = ! m_doPlayBack;
        if(m_doPlayBack) reverseButton.setIcon(backImage);
        else reverseButton.setIcon(forwardImage);
        WORLD.playback(WORLD.time());
        repaint();
        m_doRedo = true;
      }});

    panel.add(reverseButton);
    panel.add(stopButton);
    return stopButton;
  }



  private void addTimeField(JComponent panel, final JButton button) {
    panel.add(new JLabel(" Time "));
    timeField.setPreferredSize(new Dimension(30, 20));
    timeField.setMaximumSize(new Dimension(30, 20));
    timeField.setHorizontalAlignment(JTextField.RIGHT);
    timeField.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  try {
        int time = Integer.parseInt(timeField.getText());
        timeSlider.setValue(time);
	    WORLD.playback(time);
	    m_doRedo = true;
	    button.setText("O");
	    m_isWaiting = false;
	  } catch(NumberFormatException nfe) { }
	}});
    timeSlider.setPreferredSize(new Dimension(90, 30));
    timeSlider.setMaximumSize(new Dimension(90, 30));
    timeSlider.setToolTipText(""+timeSlider.getValue());
    timeSlider.addChangeListener(new ChangeListener() {
	public void stateChanged(ChangeEvent e) {
      int time = timeSlider.getValue();
      timeSlider.setToolTipText(""+time);
      timeField.setText(""+time);
      if(timeSlider.hasFocus()) WORLD.playback(time);
	}});
    panel.add(timeField);
    panel.add(timeSlider);
  }
  ///}

  private void addAttendingIdField(JComponent panel) {
    panel.add(new JLabel("    AttentingID "));
    panel.add(attendingIdField);
    attendingIdField.setMaximumSize(new Dimension(55, 20));
    attendingIdField.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  int id = -1;
	  try { id = Integer.parseInt(attendingIdField.getText()); }
	  catch(NumberFormatException nfe) { }
	  if (WORLD.get(id) == null) {
	    id = -1;
	    attendingIdField.setText("not exist");
	  }
	  if (id == m_attendingId)
	    return;
	  m_attendingId = id;
	  m_wasChangedBackground |= true;
	  repaint();
	}});
  }


  private void addIntervalEachCycleSlider(JComponent panel) {
    panel.add(new JLabel("  Interval"));
    final JSlider intervalEachCycleSlider
      = new JSlider(JSlider.HORIZONTAL, 100, 5000, VIEWER_DEFAULT_INTERVAL_EACH_CYCLE);
    intervalEachCycleSlider.setPreferredSize(new Dimension(100, 30));
    intervalEachCycleSlider.setMaximumSize(new Dimension(100, 30));
    panel.add(intervalEachCycleSlider);
    ToolTipManager tipMan = ToolTipManager.sharedInstance();
    tipMan.setInitialDelay(0);
    tipMan.setDismissDelay(1000);
    intervalEachCycleSlider.setToolTipText(""+intervalEachCycleSlider.getValue()+"ms/turn");
    intervalEachCycleSlider.addChangeListener(new ChangeListener() {
	public void stateChanged(ChangeEvent e) {
	  m_intervalEachCycle = ((JSlider) e.getSource()).getValue();
          intervalEachCycleSlider.setToolTipText(""+m_intervalEachCycle+"ms/cycle");
	}});
  }

  private void setScrollBar(JPanel panel) {
    AdjustmentListener adl = new AdjustmentListener() {
      public void adjustmentValueChanged(AdjustmentEvent e) {
        scrollBarValueChange((JScrollBar)e.getSource());
      }
    };
    panel.add(m_vScroll, BorderLayout.EAST);
    panel.add(m_hScroll, BorderLayout.SOUTH);
    m_vScroll.setVisible(false);  m_hScroll.setVisible(false);
    m_vScroll.addAdjustmentListener(adl);
    m_hScroll.addAdjustmentListener(adl);
  }
  private void setScrollCondition(boolean isVertical, int max, boolean isVisible) {
    setScrollCondition(isVertical, max, -1, isVisible);
  }
  private void setScrollCondition(boolean isVertical, int max, int value, boolean isVisible) {
    JScrollBar bar = isVertical? m_vScroll: m_hScroll;
    bar.setMaximum(max);
    bar.setVisible(isVisible);
    if(value >= 0) bar.setValue(value);
  }
  private void scrollBarValueChange(JScrollBar source){
    boolean isVertical = source == m_vScroll;
    if(isVertical) m_paintField.setOffsetY(source.getMaximum()-source.getValue());
    else m_paintField.setOffsetX(source.getValue());
  }
  ///}

  private void addDoAnimateCheckBox(JComponent panel) {
    panel.add(new JLabel("    "));
    final JCheckBox doAnimateCheckBox = new JCheckBox("Animaition", true);
    panel.add(doAnimateCheckBox);
    doAnimateCheckBox.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  m_doAnimation = !m_doAnimation;
	}});
  }

  private JPanel statusPanel() {
    JPanel panel = new JPanel(new FlowLayout(FlowLayout.CENTER));
    panel.setPreferredSize(new Dimension(VIEWER_WIDTH, 20));
    String name = Main.teamName();
    if (name instanceof String)
      panel.add(new JLabel("Team: " + name));
    panel.setBackground(new Color(100, 120, 160));
    return panel;
  }

  private class PaintField extends JComponent {
    private int m_xbase;
    private int m_ybase;
    private double m_ratio;
    private double m_magnification = 1.0d;  /// add 19/05/02{
    private int m_xoffset = 0;
    private int m_yoffset = 0;
    private Point m_startDrag = null;
    ///}

    public PaintField() {
      addMouseListener(mouseListener());
      addMouseMotionListener(mouseMotionListener());
    }

    public void paint(Graphics g) {
      paintBackground(g);
      paintFires(g);
      paintActions(g);
      paintMovingObjects(g);
      paintBlockades(g);
      paintScore(g);
      paintAttentedObject(g);
      paintSelectedObjects(g);
      paintClickedObjects(g);
    }

    private int m_lastPaintBackground = 0;

    private void paintScore(Graphics g) {
      int sx = 1;//getWidth() - 130;
      int sy = 10;//getHeight() - 5;
      g.setColor(Color.white);
      g.fillRect(sx-1, sy-10, 134, 12);
      g.setColor(Color.BLACK);
      String show = WORLD.score() + "";
      g.drawString(show, sx, sy);
    }

    private void paintBackground(Graphics g) {
      setBaseAndRatio();
      if (m_motionlessObjectsImage == null
	  || m_wasChangedBackground
	  || m_lastPaintBackground <= 3  &&  m_lastPaintBackground != WORLD.time()) {
	m_wasChangedBackground = false;
	m_lastPaintBackground = WORLD.time();
	m_motionlessObjectsImage = createImage(getWidth(), getHeight());
	Graphics mg = m_motionlessObjectsImage.getGraphics();
	mg.setColor(Confing.colorBackGround());
	mg.fillRect(0, 0, getWidth(), getHeight());

	Iterator it = WORLD.buildingList.iterator();
	while (it.hasNext())
	  paintBuildingWithState(mg, (Building) it.next());
	it = WORLD.roadList.iterator();
	while (it.hasNext()) {
	  mg.setColor(Confing.colorRoadHead());
	  paintRoad(mg, (Road) it.next());
	}
	paintBackGroundOfAttentedObject(mg);
      }
      g.drawImage(m_motionlessObjectsImage, 0, 0, this);
    }

    private void setBaseAndRatio() {
      double preRatio = m_ratio;
      int dw = WORLD.maxX() - WORLD.minX();
      int dh = WORLD.maxY() - WORLD.minY();
      m_ratio = Math.min((getWidth()  - VIEWER_GAP) / (double) dw,
			 (getHeight() - VIEWER_GAP) / (double) dh);
      m_xbase = (int) ((getWidth()  - dw * m_ratio) / 2 - WORLD.minX() * m_ratio);
      m_ybase = (int) ((getHeight() - dh * m_ratio) / 2 - WORLD.minY() * m_ratio);
      m_wasChangedBackground |= m_ratio != preRatio;
    }


    public  void setMagnification(double m) {
      if(m > 0.0) m_magnification = m; else assert false;
      if(m <= 1.0){
        m_xoffset = 0;  m_yoffset = 0;
        setScrollCondition(false, 0, 0, false);
        setScrollCondition(true,  0, 0, false);
      }
      m_wasChangedBackground = true;
    }
    public  double getMagnification()         { return m_magnification; }
    public  void setOffsetX(int x) { m_xoffset = x * VIEWER_SCROLL_LENGTH; m_wasChangedBackground = true; }
    public  void setOffsetY(int y) { m_yoffset = y * VIEWER_SCROLL_LENGTH; m_wasChangedBackground = true; }
    private int x(int x) { return               (int) (m_magnification * ((x - m_xoffset) * m_ratio + m_xbase)); }
    private int y(int y) { return getHeight() - (int) (m_magnification * ((y - m_yoffset) * m_ratio + m_ybase)); }
    ///}
    private void paintBuilding(Graphics g, Building bldg) {
      paintColoredShape(g, shape(bldg));
    }

    private void paintColoredShape(Graphics g, Polygon shape) {
      g.fillPolygon(shape);
      g.setColor(g.getColor().darker().darker());
      g.drawPolygon(shape);
    }

    private Polygon shape(Building bldg) {
      Polygon shape = new Polygon();
      int[] apexes = bldg.buildingApexes();
      for (int i = 0;  i < apexes.length;  i += 2)
	shape.addPoint(x(apexes[i]), y(apexes[i + 1]));
      return shape;
    }

    private void paintBuildingWithState(Graphics g, Building bldg) {
      g.setColor(Confing.colorBuilding());
      Iterator it = bldg.entrances().iterator();
      while (it.hasNext()) {
	MotionlessObject mv = (MotionlessObject) it.next();
	g.drawLine(x(bldg.x()), y(bldg.y()), x(mv.x()), y(mv.y()));
      }

      Polygon shape = shape(bldg);
      paintBroken(g, bldg, shape);

      g.setColor(Color.gray.darker());
      g.drawPolygon(shape);

      paintBuildingType(g, bldg, shape);
    }

    private void paintBroken(Graphics g, Building bldg, Polygon shape) {
      if (bldg.brokenness() == 0  ||  !bldg.isNoBurned())
	return;
      int b = Math.max(0, 135 - bldg.brokenness() / 4);
      g.setColor(new Color(b, b, b));
      paintColoredShape(g, shape);
    }

    private void paintBuildingType(Graphics g, Building bldg, Polygon shape) {
      if (bldg.type() == TYPE_BUILDING) return;
      else if (bldg.type() == TYPE_REFUGE) g.setColor(Confing.colorRefuge());
      else                            g.setColor(Confing.colorAmbulanceCenter());
      paintColoredShape(g, shape);
    }

    private void paintFires(Graphics g) {
      Iterator it = WORLD.buildingList.iterator();
      while (it.hasNext())
	paintFire(g, (Building) it.next());
    }

    private void paintFire(Graphics g, Building bldg) {
      if (bldg.isNoBurned())
	return;
      g.setColor(Confing.colorBuildingFire(bldg.fieryness()));
      paintBuilding(g, bldg);
    }

    private void paintRoad(Graphics g, Road road) {
      PointObject h = road.head();
      PointObject t = road.tail();
      g.fillPolygon(shape(road));
      g.setColor(Color.gray.darker());
      paintNode(g, h, 0);
      paintNode(g, t, 0);
    }

    private Polygon shape(Road road) {
      PointObject h = road.head();
      PointObject t = road.tail();
      int width = Math.min(VIEWER_ROAD_MAX_WIDTH, road.width());
      int dx = (h.y() - t.y()) * width / road.length() / 2;
      int dy = (h.x() - t.x()) * width / road.length() / 2;
      Polygon shape = new Polygon();
      shape.addPoint(x(h.x() - dx), y(h.y() + dy));
      shape.addPoint(x(h.x() + dx), y(h.y() - dy));
      shape.addPoint(x(t.x() + dx), y(t.y() - dy));
      shape.addPoint(x(t.x() - dx), y(t.y() + dy));
      return shape;
    }

    private void paintNode(Graphics g, PointObject po, int dsize) {
      int size = VIEWER_NODE_SIZE + dsize;
      g.fillOval(x(po.x()) - size / 2, y(po.y()) - size / 2, size, size);
    }

    private void paintBlockades(Graphics g) {
      Iterator it = WORLD.roadList.iterator();
      while (it.hasNext())
	paintBlockade(g, (Road) it.next());
    }

    private void paintBlockade(Graphics g, Road road) {
      if (road.blockedLines() == 0)
	return;
      final int B = VIEWER_BLOCKADE_SIZE;
      int x = x(road.x()) - B / 2;
      int y = y(road.y()) - B / 2;
      g.setColor(road.aliveLinesToHead() > 0  ||  road.aliveLinesToTail() > 0
		 ? Color.gray.darker() : Color.black);
      g.drawLine(x, y,     x + B, y + B);
      g.drawLine(x, y + B, x + B, y    );
      x ++;
      g.drawLine(x, y,     x + B, y + B);
      g.drawLine(x, y + B, x + B, y    );
    }

    private void paintMovingObjects(Graphics g) {
      paintMovingObjects(g, WORLD.civilianList);
      paintMovingObjects(g, WORLD.policeForceList);
      paintMovingObjects(g, WORLD.fireBrigadeList);
      paintMovingObjects(g, WORLD.ambulanceTeamList);
      paintLoadedMvObjs(g);
    }

    private void paintMovingObjects(Graphics g, Collection objs) {
      Iterator it = objs.iterator();
      while (it.hasNext()) {
	MovingObject mv = (MovingObject) it.next();
	if (mv.position() instanceof MotionlessObject)
	  paintColoredMovingObject(g, (MovingObject) mv, 0);
      }
    }

    private void paintLoadedMvObjs(Graphics g) {
      Iterator it = WORLD.movingObjectList.iterator();
      while (it.hasNext()) {
	MovingObject mv = (MovingObject) it.next();
	if (mv.position() instanceof MotionlessObject)
	  continue;
	paintColoredMovingObject(g, (MovingObject) mv.position(), +2);
	paintColoredMovingObject(g, mv,                           -2);
      }
    }

    private void paintColoredMovingObject(Graphics g, MovingObject mv, int dsize) {
      Color color;

      switch (mv.type()) {
	case TYPE_CIVILIAN:        color = Confing.colorCivilian();  break;
	case TYPE_AMBULANCE_TEAM:  color = Confing.colorAmbulance();  break;
	case TYPE_FIRE_BRIGADE:    color = Confing.colorFire();    break;
	case TYPE_POLICE_FORCE:    color = Confing.colorPolice(); break;
	default: assert false; throw new Error();
      }
      int hp = ((Humanoid) mv).hp();
      if (hp == 0)
	color = Color.black;
      else
	for (int i = 10000 - hp;  i > 0;  i -= 3500) // 10000: max hp
	  color = color.darker();
      g.setColor(color);

      paintMovingObject(g, mv, dsize);
    }

    private void paintMovingObject(Graphics g, MovingObject mv, int dsize) {
      int x = x(mv.x());
      int y = y(mv.y());
      int h = paintingSizeOfMovingObject(mv) + dsize;
      g.fillOval(x - h / 2, y - h / 2, h, h);
      g.setColor(g.getColor().darker().darker().darker());
      g.drawOval(x - h / 2, y - h / 2, h, h);
    }

    private int paintingSizeOfMovingObject(MovingObject mv) {
      return VIEWER_HUMANOID_SIZE - ((mv instanceof Civilian) ? 1 : 0);
    }

    private void paintAttentedObject(Graphics g) {
      RealObject obj = WORLD.get(m_attendingId);
      if (obj == null)
	return;
      g.setColor(Color.cyan);
      paintObject(g, obj);
      attendingIdField.selectAll();
    }

    private void paintSelectedObject(Graphics g, Iterator iter, Color c) {
      while (iter.hasNext()) {
        int id = ((Integer)iter.next()).intValue();
        RealObject obj = WORLD.get(id);
        if(obj == null) continue;
        g.setColor(c);
        paintObject(g, obj);
      }
    }

    private void paintSelectedObjects(Graphics g) {
      paintSelectedObject(g, AGENT_VIEWER.m_selectedIdList.iterator(), VIEWER_SELECTED_COLOR);
      paintSelectedObject(g, AGENT_VIEWER.m_selectedParentIdList.iterator(), VIEWER_SELECTED_COLOR);
      paintSelectedObject(g, STATUS_VIEWER.m_selectedIdList.iterator(), VIEWER_SELECTED_COLOR);
    }

    private void paintClickedObjects(Graphics g) {
      for(int i=0;i<clickedObjects.size();++i)
        g.drawString(((RealObject)clickedObjects.get(i)).toString(), 3, getHeight()-(5+i*13));
    }

    private void paintBackGroundOfAttentedObject(Graphics g) {
      RealObject obj = WORLD.get(m_attendingId);
      if (obj == null  ||  obj instanceof MovingObject)
	return;
      g.setColor(new Color(200, 100, 200));
      final int S = VIEWER_BACKGROUND_OF_ATTENTED_OBJECT_SIZE;
      g.fillOval(x(obj.x()) - S / 2, y(obj.y()) - S / 2, S, S);
    }

    private void paintObject(Graphics g, Object obj) {
      if      (obj instanceof Road) paintRoad(g, (Road) obj);
      else if (obj instanceof Node) paintNode(g, (Node) obj, +2);
      else if (obj instanceof Building) paintBuilding(g, (Building) obj);
      else if (obj instanceof MovingObject) paintMovingObject(g, (MovingObject) obj, +2);
      else assert false;
    }

    //public final java.util.List extinguishTargetList = new ArrayList();
    private void paintActions(Graphics g) {
      //extinguishTargetList.clear();
      Iterator it = WORLD.movingObjectList.iterator();
      while (it.hasNext()) {
        MovingObject mv = (MovingObject) it.next();
        switch (mv.actionType()) {
          default: break;
          case AK_LOAD:  case AK_UNLOAD: break;  case AK_RESCUE:
            g.setColor(new Color(170, 170, 150));
            paintMovingObject(g, mv, + 6);
            break;
          case AK_EXTINGUISH:
            int target = -1;
            try { target = mv.actionData().readInt(); }
            catch (IOException e) { assert false; }
            Building b = (Building) WORLD.get(target);
            g.setColor(Confing.colorActionExtinguish());
            //if(!extinguishTargetList.contains(b)) extinguishTargetList.add(b);
            g.drawLine(x(mv.x()), y(mv.y()), x(b.x()), y(b.y()));
            break;
          case AK_CLEAR:
            g.setColor(new Color(150, 150, 70));
            paintMovingObject(g, mv, +6);
            break;
        }
      }
    }

    private MouseListener mouseListener() {
      return new MouseListener() {
	  public void mouseClicked(MouseEvent e) {
            clickedObjects.clear();
	    Iterator it = objectsAt(e.getX(), e.getY()).iterator();
        if(it.hasNext()) System.out.println();
	    while (it.hasNext()) {
              RealObject tmp =  (RealObject) it.next();
              System.out.println(tmp.toString());
              clickedObjects.add(tmp);
            }
            paint(getGraphics());
	  }
	  public void mouseEntered(MouseEvent e)  { }
	  public void mouseExited(MouseEvent e)   { }
	  public void mousePressed(MouseEvent e)  { m_startDrag = e.getPoint(); }
	  public void mouseReleased(MouseEvent e) {
            int rx = e.getX(),  ry = e.getY();
            int sx = m_startDrag.x,  sy = m_startDrag.y;
            double w  = Math.abs(sx - rx),  h = Math.abs(sy - ry);
            if(sx > rx) sx = rx;
            if(sy > ry) sy = ry;
            if(w < 10  ||  h < 10) return;
            double magnification = (getWidth() / w  +  getHeight() / h) / 2;
            if(magnification < 16) {
              m_xoffset = (int)((sx - 20 / magnification) / m_ratio);
              m_yoffset = (int)((sy + h/2 - 20 / magnification) / m_ratio);
              setScrollCondition(false, (WORLD.maxX()-WORLD.minX())/VIEWER_SCROLL_LENGTH, (int)(m_xoffset / VIEWER_SCROLL_LENGTH), true);
              setScrollCondition(true,  (WORLD.maxY()-WORLD.minY())/VIEWER_SCROLL_LENGTH, (int)(m_yoffset / VIEWER_SCROLL_LENGTH), true);
              setMagnification(magnification);
            }
          }
	};
    }
    private MouseMotionListener mouseMotionListener() {
      return new MouseMotionListener() {
        public void mouseMoved(MouseEvent e) {}
        public void mouseDragged(MouseEvent e) {
          if(m_startDrag == null) return;
          int dx = e.getX(),  dy = e.getY();
          int sx = m_startDrag.x,  sy = m_startDrag.y;
          int w  = Math.abs(dx - sx),  h = Math.abs(dy - sy);
          boolean xflag = false,  yflag = false;
          if(dx < sx) { int tmp=sx;  sx = dx; dx = tmp;  xflag = true; }
          if(dy < sy) { int tmp=sy;  sy = dy; dy = tmp;  yflag = true; }
          Graphics g = getGraphics();
          g.setColor(Color.black);
          g.drawRect(sx, sy, w, h);
        }
      };
    }

    private Collection objectsAt(int x, int y) {
      Collection objs = new ArrayList();
      Iterator it = WORLD.idObjMap.values().iterator();
      while (it.hasNext()) {
	RealObject obj = (RealObject) it.next();
	if (obj instanceof Node  ||  obj instanceof MovingObject) {
	  int dx = x - x(obj.x());
	  int dy = y - y(obj.y());
	  int d = ((obj instanceof Node) ? VIEWER_NODE_SIZE : paintingSizeOfMovingObject((MovingObject) obj)) / 2 + 1;
	  if (dx * dx + dy * dy <= d * d)
	    objs.add(obj);
	} else if (obj instanceof Building) {
	  if (shape((Building) obj).contains(x, y))
	    objs.add(obj);
	} else if (obj instanceof Road) {
	  if (shape((Road) obj).contains(x, y))
	    objs.add(obj);
	}
      }
      return objs;
    }
  }

  private boolean m_doRedo = false;

  public void animate() {
    redo:
    while (true) {
      m_doRedo = false;
      WORLD.extractNextPositionPropertys();
      for (int i = numOfExposuresEachCycle();  i > 0  &&  m_doAnimation;  i --) {
	Iterator it = WORLD.movingObjectList.iterator();
	while (it.hasNext())
	  ((MovingObject) it.next()).move();
	repaint();
	waitToPushStopButton();
	if (m_doRedo)
	  continue redo;
      }
      return;
    }
  }

  public int numOfExposuresEachCycle() { return m_intervalEachCycle / VIEWER_INTERVAL_EACH_EXPOSURE; }

  private void waitToPushStopButton() {
    do {
      try { Thread.sleep(VIEWER_INTERVAL_EACH_EXPOSURE); }
      catch (InterruptedException e) { assert false; }
    } while (m_isWaiting);
  }

  public void repaintWithWaiting() {
    repaint();
    for (int i = 0;  i < numOfExposuresEachCycle();  i ++)
      waitToPushStopButton();
  }

  //// close when 'x' clicked
  class ViewerWindowAdapter extends WindowAdapter {
      public void windowClosing(WindowEvent e) {
        try{
          Confing.writeConfingFile();
        } catch(IOException ioe) {}
        System.exit(0);
      }
  }
}

/*
 * 03/06/02 changed all
*/
