package debugger;

import java.io.*;
import java.util.*;
import debugger.object.*;

public class WorldModel implements Constants {
  public final HashMap idObjMap = new HashMap();

  public final ArrayList nodeList          = new ArrayList();
  public final ArrayList roadList          = new ArrayList();
  public final ArrayList buildingList      = new ArrayList();
  public final ArrayList movingObjectList  = new ArrayList();
  public final ArrayList ambulanceTeamList = new ArrayList();
  public final ArrayList fireBrigadeList   = new ArrayList();
  public final ArrayList policeForceList   = new ArrayList();
  public final ArrayList civilianList      = new ArrayList();

  private int m_time = 0;
  public int time() { return m_time; }

  public void add(RealObject obj) {
    idObjMap.put(new Integer(obj.id), obj);
    if      (obj instanceof Node)     nodeList    .add(obj);
    else if (obj instanceof Road)     roadList    .add(obj);
    else if (obj instanceof Building) buildingList.add(obj);
    else if (obj instanceof MovingObject) {
            movingObjectList.add(obj);
            switch (obj.type()) {
            default: break;
            case TYPE_AMBULANCE_TEAM:  ambulanceTeamList.add(obj); break;
            case TYPE_FIRE_BRIGADE:    fireBrigadeList.add(obj);   break;
            case TYPE_POLICE_FORCE:    policeForceList.add(obj);   break;
            case TYPE_CIVILIAN:        civilianList.add(obj);      break;
            }
    }
  }

  public RealObject get(int id) {
    return (RealObject) idObjMap.get(new Integer(id));
  }

  public void update(DataInputStream dis, int time) {
    m_time = time;
    VIEWER.timeField.setText(String.valueOf(time));
    if(!VIEWER.timeSlider.hasFocus())VIEWER.timeSlider.setValue(time);
   try {
      int type = dis.readInt();
      while (type != TYPE_NULL) {
        int size = dis.readInt();
        int id = dis.readInt();
        RescueObject obj = get(id);
        if (obj == null) {
          obj = newRescueObject(type, id);
          if (obj instanceof RealObject)
            add((RealObject) obj);
        }
        int propType = dis.readInt();
        while (propType != PROPERTY_NULL) {
          obj.input(propType, getProperty(propType, dis));
          propType = dis.readInt();
        }
        type = dis.readInt();
      }
    } catch (IOException e) { assert false : e; }
    if(time == 0) initStatus();  // add 04/06/02
  }

  private RescueObject newRescueObject(int type, int id) {
    RescueObject obj;
    switch (type) {
      case TYPE_WORLD:            obj = new World(id);           break;
      case TYPE_RIVER:            obj = new River(id);           break;
      case TYPE_RIVER_NODE:       obj = new RiverNode(id);       break;
      case TYPE_ROAD:             obj = new Road(id);            break;
      case TYPE_NODE:             obj = new Node(id);            break;
      case TYPE_BUILDING:         obj = new Building(id);        break;
      case TYPE_AMBULANCE_CENTER: obj = new AmbulanceCenter(id); break;
      case TYPE_FIRE_STATION:     obj = new FireStation(id);     break;
      case TYPE_POLICE_OFFICE:    obj = new PoliceOffice(id);    break;
      case TYPE_REFUGE:           obj = new Refuge(id);          break;
      case TYPE_CIVILIAN:         obj = new Civilian(id);        break;
      case TYPE_AMBULANCE_TEAM:   obj = new AmbulanceTeam(id);   break;
      case TYPE_FIRE_BRIGADE:     obj = new FireBrigade(id);     break;
      case TYPE_POLICE_FORCE:     obj = new PoliceForce(id);     break;
      case TYPE_CAR:              obj = new Car(id);             break;
      default: assert false : "illeagle object type" + type; throw new Error();
    }
    return obj;
  }

  private int[] getProperty(int type, DataInputStream dis) throws IOException {
    int size = dis.readInt();
    int[] val = null;
    switch (type) {
    case PROPERTY_START_TIME:
    case PROPERTY_LONGITUDE:
    case PROPERTY_LATITUDE:
    case PROPERTY_WIND_FORCE:
    case PROPERTY_WIND_DIRECTION:
    case PROPERTY_X:
    case PROPERTY_Y:
    case PROPERTY_DIRECTION:
    case PROPERTY_POSITION:
    case PROPERTY_POSITION_EXTRA:
    case PROPERTY_STAMINA:
    case PROPERTY_HP:
    case PROPERTY_DAMAGE:
    case PROPERTY_BURIEDNESS:
    case PROPERTY_FLOORS:
    case PROPERTY_BUILDING_ATTRIBUTES:
    case PROPERTY_IGNITION:
    case PROPERTY_BROKENNESS:
    case PROPERTY_FIERYNESS:
    case PROPERTY_BUILDING_TEMPERATURE:
            //		case PROPERTY_BUILDING_SHAPE_ID:
    case PROPERTY_BUILDING_CODE:
    case PROPERTY_BUILDING_AREA_GROUND:
    case PROPERTY_BUILDING_AREA_TOTAL:
    case PROPERTY_WATER_QUANTITY:
            //		case PROPERTY_STRETCHED_LENGTH:
    case PROPERTY_HEAD:
    case PROPERTY_TAIL:
    case PROPERTY_LENGTH:
    case PROPERTY_ROAD_KIND:
    case PROPERTY_CARS_PASS_TO_HEAD:
    case PROPERTY_CARS_PASS_TO_TAIL:
    case PROPERTY_HUMANS_PASS_TO_HEAD:
    case PROPERTY_HUMANS_PASS_TO_TAIL:
    case PROPERTY_WIDTH:
    case PROPERTY_BLOCK:
    case PROPERTY_REPAIR_COST:
    case PROPERTY_MEDIAN_STRIP:
    case PROPERTY_LINES_TO_HEAD:
    case PROPERTY_LINES_TO_TAIL:
    case PROPERTY_WIDTH_FOR_WALKERS:
    case PROPERTY_SIGNAL:
            val = new int[] {dis.readInt()};
            break;
    case PROPERTY_EDGES:
    case PROPERTY_SIGNAL_TIMING:
    case PROPERTY_SHORTCUT_TO_TURN:
    case PROPERTY_POCKET_TO_TURN_ACROSS:
    case PROPERTY_POSITION_HISTORY:
    case PROPERTY_ENTRANCES:
    case PROPERTY_BUILDING_APEXES:
            val = new int[dis.readInt()];
            for (int i=0;i<val.length;++i) val[i] = dis.readInt();
            break;
    default:
            System.err.println("Unrecognised property: "+type);
            dis.skip(size);
            break;
    }
    return val;
  }

  private int m_minX = Integer.MAX_VALUE;
  private int m_minY = Integer.MAX_VALUE;
  private int m_maxX = Integer.MIN_VALUE;
  private int m_maxY = Integer.MIN_VALUE;
  public int minX() { return m_minX; }
  public int maxX() { return m_maxX; }
  public int minY() { return m_minY; }
  public int maxY() { return m_maxY; }

  public void setWorldRange() {
    Iterator it = idObjMap.values().iterator();
    while (it.hasNext()) {
      Object obj = it.next();
      if (!(obj instanceof PointObject))
	continue;
      PointObject po = (PointObject) obj;
      if (m_minX > po.x()) m_minX = po.x();
      if (m_maxX < po.x()) m_maxX = po.x();
      if (m_minY > po.y()) m_minY = po.y();
      if (m_maxY < po.y()) m_maxY = po.y();
    }
  }

  private IO io() { return Main.debugger().io; }

  public void extractNextPositionPropertys() {
    if (!io().hasUpdateData(m_time + 1))
            return;
    DataInputStream dis = io().updateData(m_time + 1);
    try{
    int type = dis.readInt();
    while (type != TYPE_NULL) {
            int size = dis.readInt();
            int id = dis.readInt();
            RescueObject obj = get(id);
            int pos = 0, posEx = 0, route[] = new int[0];
            MovingObject mv = null;
            if (obj instanceof MovingObject) {
                    mv = (MovingObject) obj;
                    pos   = mv.motionlessPosition().id;
                    posEx = mv.positionExtra();
            }
            int propType = dis.readInt();
            while (propType != PROPERTY_NULL) {
                    int[] val = getProperty(propType, dis);
                    switch (propType) {
                    default: break;
                    case PROPERTY_POSITION:         pos   = val[0]; break;
                    case PROPERTY_POSITION_EXTRA:   posEx = val[0]; break;
                    case PROPERTY_POSITION_HISTORY: route = val;    break;
                    }
                    propType = dis.readInt();
            }
            if (obj instanceof MovingObject)
                    mv.prepareForAnimation(pos, posEx, route);
            type = dis.readInt();
    }
    } catch (IOException e) { assert false : e; }
  }

  public void progress() {
//    if(VIEWER.doPlayBack()) { if( m_time > 0 ) playback(m_time - 1); }
//    else
    playback(m_time + 1);
  }

  public void playback(int time) {
    for (int i = (time > m_time) ? m_time : 0;
             i <= time  &&  io().hasUpdateData(i);
             i ++) {
            //			System.out.println("Updating timestep "+i);
            update(io().updateData(i), i);
    }
  }

  public void parseCommands() {
    DataInputStream dis = io().commandsData(m_time + 1);
    try {
    int time = dis.readInt();
    int command = dis.readInt();
    while (command != HEADER_NULL) {
            int size = dis.readInt();
            if (size != 0) {
                    int senderId = dis.readInt();
                    byte[] content = new byte[size-4];
                    dis.read(content);
                    RealObject sender = get(senderId);
                    switch (command) {
                    case AK_MOVE:
                    case AK_RESCUE:
                    case AK_LOAD:
                    case AK_UNLOAD:
                    case AK_EXTINGUISH:
                    case AK_CLEAR:
                            sender.setAction(command, content);
                            break;
                    case AK_TELL:
                    case AK_SAY:
                            sender.setCommunication(command, content);
                            break;
                    default:
                            break;
                    }
            }
            command = dis.readInt();
    }
    } catch (IOException e) { assert false : e; }
  }

  //// Atributes for score
  private int m_initialAllHp = 0;
  private int m_totalArea    = 0;
  private int m_dead         = 0;
  ////
  private void initStatus() {
    m_initialAllHp = allHp();
    m_totalArea    = area(false);
  }
  public int allHp() {
    int ret = 0;
    int dead = 0;
    Iterator iter = movingObjectList.iterator();
    while (iter.hasNext()) {
      Object item = iter.next();
      if(item instanceof Humanoid){
        Humanoid h = (Humanoid) item;
        int hp = h.hp();
        ret += hp;
        if(hp <= 0) dead++;
      }
    }
    m_dead = dead;
    return ret;
  }
  public int area(boolean isBurnedOnly) {
    int ret = 0;
    Iterator iter = buildingList.iterator();
    while (iter.hasNext()) {
      Building b = (Building)iter.next();
      if(isBurnedOnly){
        //if(b.fieryness() > 0) ret += b.buildingAreaTotal();
        if((b.fieryness() > 1 && b.fieryness() <= 4) || b.fieryness() > 6)
          ret += b.buildingAreaTotal();
      }else ret += b.buildingAreaTotal();
    }
    return ret;
  }

  public double score() {
    int allHp = allHp();
    int burned= area(true);
    int alive = movingObjectList.size() - m_dead;
    //System.out.println("dead:"+m_dead+"all:"+m_initialAllHp+" call:"+allHp+" alarea:"+m_totalArea+" burn:"+(m_totalArea-burned));
    return (alive + allHp / m_initialAllHp) *
           Math.sqrt(((double)(m_totalArea - burned)) / m_totalArea);
  }
}

/**
 * 03/06/02 Added
 * double score(), Atributes for score
 *
 * 26/05/02 Changed
 * void progress()
 *
 **/
