package debugger.view;

import debugger.Constants;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class AgentIDViewer extends JFrame implements Constants{
  public  final java.util.List m_selectedIdList = new ArrayList();
  public  final java.util.List m_selectedParentIdList = new ArrayList();
  private File m_logfile = null;
  public  File getFile() { return m_logfile; }
  private HashMap m_timeHashData = new HashMap();
  private InputStream m_fis;
  private BufferedReader m_dis;
  private int m_time = 0, m_maxtime = 0;
  private boolean m_doSync = true;
  JToolBar m_toolBar = new JToolBar();
  JLabel jLabel1 = new JLabel();
  JTextField m_timeField = new JTextField();
  JSplitPane jSplitPane1 = new JSplitPane();
  JScrollPane m_scrollPane1 = new JScrollPane();
  JScrollPane m_scrollPane2 = new JScrollPane();
  JList m_idList = new JList();
  JList m_collectionList = new JList();
  JSlider m_timeSlider = new JSlider();
  JCheckBox m_syncView = new JCheckBox();

  public AgentIDViewer() {
    super("AGENT LOG VIEWER");
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
    pack();
    this.setBounds(600, 100, 400, 250);
  }
  private void jbInit() throws Exception {
    m_timeSlider.setMajorTickSpacing(100);
    m_timeSlider.setMaximum(310);
    m_timeSlider.setMinorTickSpacing(10);
    m_timeSlider.setMaximumSize(new Dimension(32767, 30));
    m_timeSlider.setMinimumSize(new Dimension(36, 30));
    m_timeSlider.setPreferredSize(new Dimension(150, 30));
    m_timeSlider.setPaintTicks(true);
    m_timeSlider.setValue(0);
    m_timeSlider.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        int value = m_timeSlider.getValue();
        int ptime = m_time;
        m_timeField.setText(""+value);
        m_time = value;
        if(m_time != ptime) {
          setListValues();
          if(m_doSync) WORLD.playback(m_time);
        }
      }
    });
    m_timeField.setHorizontalAlignment(SwingConstants.RIGHT);
    m_timeField.setMinimumSize(new Dimension(60, 30));
    jSplitPane1.setPreferredSize(new Dimension(200, 134));
    m_syncView.setToolTipText("Time=Viewer\'s Time");
    m_syncView.setText("syncView");
    m_syncView.setSelected(true);
    m_toolBar.add(m_syncView, null);
    m_toolBar.add(jLabel1, null);
    m_toolBar.add(m_timeField, null);
    m_toolBar.add(m_timeSlider, null);
    this.getContentPane().add(jSplitPane1, BorderLayout.CENTER);
    jLabel1.setText("time: ");
    m_timeField.setText("000");
    this.getContentPane().add(m_toolBar, BorderLayout.NORTH);
    jSplitPane1.add(m_scrollPane1, JSplitPane.LEFT);
    jSplitPane1.add(m_scrollPane2, JSplitPane.RIGHT);
    m_scrollPane2.getViewport().add(m_collectionList, null);
    m_scrollPane1.getViewport().add(m_idList, null);
    jSplitPane1.setDividerLocation(325);
    //// Event
    m_timeField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try{
          int m_time = Integer.parseInt(m_timeField.getText());
          m_timeSlider.setValue(m_time);
        } catch(Exception exc){}
      }
    });
      //// checkBox
    m_syncView.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try{
          m_doSync = m_syncView.isSelected();
          if(m_doSync) m_syncView.setToolTipText("Viewer'sTime = time");
          else m_syncView.setToolTipText("Viewer'sTime != time");
        } catch(Exception exc){}
      }
    });
      //// list
    ListSelectionListener lsl = new ListSelectionListener(){
      public void valueChanged(ListSelectionEvent e) {
        JList ls = (JList) e.getSource();
        Object selected[] = ls.getSelectedValues();
        m_selectedIdList.clear();
        m_selectedParentIdList.clear();
        for (int i = 0; i < selected.length; i++) {
          if(ls == m_collectionList) m_selectedIdList.add(selected[i]);
          else{
            Iterator iter = ((AgentData)selected[i]).m_idList.iterator();
            while (iter.hasNext()) {
              m_selectedIdList.add(iter.next());
            }
            Integer id = new Integer(((AgentData)selected[i]).m_id);
            m_selectedIdList.add(id);
            m_selectedParentIdList.add(id);
            setCollectionListValues();
          }
        }
        VIEWER.repaint();
      }
    };
    m_collectionList.addListSelectionListener(lsl);
    m_idList.addListSelectionListener(lsl);
  }

  public boolean isSelected(int id) {
    return m_selectedIdList.contains(new Integer(id));
  }

  public void setLogfile(File log) {
    m_logfile = log;
    this.setTitle(log.getPath());
    int line = 0;
    String key = "";
    System.out.println("reading..." + log.getPath());
    try{
      m_fis = new FileInputStream(log);
      m_dis = new BufferedReader(new InputStreamReader(m_fis));
      key = m_dis.readLine();
      while(m_fis.available()>0){
        while(key != null  &&  key.indexOf("#") !=0 ){
          if(m_fis.available()<=0){
            m_fis.close();
            return;
          }
          key = m_dis.readLine();
          ++line;
        }
        if(key == null) break;
        java.util.List keyList = parseKey(key);
        Iterator iter = keyList.iterator();
        String time = (String)iter.next();
        String id = (String)iter.next();
        String collection = (String)iter.next();

        String data = m_dis.readLine();
        ++line;
        ArrayList dataList = (ArrayList)m_timeHashData.get(time);
        if(dataList == null){
          dataList = new ArrayList();
          m_timeHashData.put(time, dataList);
        }
        if(data == null) break;
        else if(data.indexOf("[") == 0){
          dataList.add(new AgentData(Integer.parseInt(id), collection, data));
        } else if(data.indexOf("#") == 0){
          dataList.add(new AgentData(Integer.parseInt(id), collection, null));
          key = data;  continue;
        }
        key = m_dis.readLine();
        ++line;
      }
    } catch(Exception e) {
      e.printStackTrace();
      System.err.println("line:"+line+" KEY:"+key);
    }
  }

  private java.util.List parseKey(String key) {
    StringTokenizer token = new StringTokenizer(key);
    String time = token.nextToken(" #");
    String id = token.nextToken(" ");
    String collection = token.nextToken(" ");
    ArrayList ret = new ArrayList(3);
    ret.add(time);  ret.add(id);  ret.add(collection);
    return ret;
  }

  public void setListValues() {
    java.util.List timeList = (java.util.List)m_timeHashData.get(new String(""+m_time));
    if(timeList == null){
      m_idList.setListData(new Object[0]);
      m_collectionList.setListData(new Object[0]);
      return;
    }
    Iterator iter = timeList.iterator();
    Vector idShow = new Vector(19);
    while (iter.hasNext()) {
      AgentData item = (AgentData)iter.next();
      idShow.add(item);
    }
    m_idList.setListData(idShow);
  }
  private void setCollectionListValues() {
    Object[] selected = m_idList.getSelectedValues();
    Vector show = new Vector();
    for (int i = 0; i < selected.length; i++) {
      Iterator iter = ((AgentData)selected[i]).m_idList.iterator();
      while (iter.hasNext()) {
        show.add(iter.next());
      }
    }
    m_collectionList.setListData(show);
  }

  public void progress() {
    if(!m_doSync || !isVisible()) return;
    m_time = WORLD.time();
    setListValues();
    m_timeSlider.setValue(m_time);
  }

  private class AgentData {
    public final int m_id;
    public final String m_collection;
    public final java.util.List m_idList = new ArrayList();
    AgentData(int id, String collection, String idList) {
      m_id = id;
      m_collection = collection;
      if(idList != null) parseData(idList);
    }
    private void parseData(String idList) {
      StringBuffer sb = new StringBuffer(idList);
      int index = sb.indexOf("id=");
      while(index > 0){
        sb = sb.delete(0, index+3);
        int space = sb.indexOf(" ");
        if(space < 0) space = sb.length();
        String id = sb.substring(0, space);
        sb.delete(0, space);
        index = sb.indexOf("id=");
        m_idList.add(new Integer(id));
      }
    }
    public String toString() {
      return "["+m_id+"     "+m_collection+"]";
    }
  }
}