package debugger.view;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import debugger.*;
import debugger.object.*;

/**
 * Message Viewer
 */

public class MessageViewer extends JFrame implements Constants{
  public static final String STR_AMBULANCE_TEAM = "AmbulanceTeam";
  public static final String STR_POLICE_FORCE  = "PoliceForce";
  public static final String STR_FIRE_BRIGADE   = "FireBrigade";
  public static final String STR_AMBULANCE_CENTER  = "AmbulanceCenter";
  public static final String STR_POLICE_OFFICE  = "PoliceOffice";
  public static final String STR_FIRE_STATION   = "FireStation";
  public static final String STR_ONE_AGENT      = "OneAgent";
  public final JTextField m_idField = new JTextField(" Input Shown ID ");
  public final PaintField m_idPaint = new PaintField();

  public MessageViewer() {
    super("Message Viewer");
    JSplitPane split = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    JSplitPane split2= new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    JPanel idPanel   = new JPanel(new BorderLayout());
    JPanel teamPanel = new JPanel(new GridLayout(1, 3));
    JPanel teamPanel2= new JPanel(new GridLayout(1, 3));
    idPanel.add(m_idField, BorderLayout.NORTH);
    idPanel.add(m_idPaint, BorderLayout.CENTER);
    split.add(idPanel);     split.add(split2);
    split2.add(teamPanel);  split2.add(teamPanel2);
    /// set team panel
    teamPanel.add(makePaintPanel(STR_AMBULANCE_TEAM));
    teamPanel.add(makePaintPanel(STR_POLICE_FORCE));
    teamPanel.add(makePaintPanel(STR_FIRE_BRIGADE));
    teamPanel2.add(makePaintPanel(STR_AMBULANCE_CENTER));
    teamPanel2.add(makePaintPanel(STR_POLICE_OFFICE));
    teamPanel2.add(makePaintPanel(STR_FIRE_STATION));
    /// Event
    m_idField.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e){
        try{
          int id = Integer.parseInt(m_idField.getText());
          m_idPaint.setPaintedId(id);
          m_idPaint.repaint();
        } catch(Exception exception) { }
      }
    });
    ///
    this.getContentPane().add(split);
    this.pack();
    this.setSize(new Dimension(600, 300));
  }

  public void progress() {
    if(!this.isVisible()) return;
  }

  private JPanel makePaintPanel(String label) {
    final JPanel panel = new JPanel(new BorderLayout());
    panel.add(new JLabel(label), BorderLayout.NORTH);
    panel.add(new PaintField(label), BorderLayout.CENTER);
    return panel;
  }

  private class PaintField extends JComponent {
    public static final int VERTICAL_MARGIN   = 10;
    public static final int HORIZONTAL_MARGIN = 5;
    private int m_id = 0;
    private int m_showKind = 0;
    private int m_rectWidth, m_rectHeight;

    public PaintField() { this(STR_ONE_AGENT); }
    public PaintField(String shown) {
      if(shown.equals(STR_AMBULANCE_TEAM)) m_showKind = TYPE_AMBULANCE_TEAM;
      else if(shown.equals(STR_POLICE_FORCE)) m_showKind = TYPE_POLICE_FORCE;
      else if(shown.equals(STR_FIRE_BRIGADE)) m_showKind = TYPE_FIRE_BRIGADE;
      else if(shown.equals(STR_AMBULANCE_CENTER)) m_showKind = TYPE_AMBULANCE_CENTER;
      else if(shown.equals(STR_POLICE_OFFICE)) m_showKind = TYPE_POLICE_OFFICE;
      else if(shown.equals(STR_FIRE_STATION)) m_showKind = TYPE_FIRE_STATION;
    }

    public void paintComponent(Graphics g) {
      setRatio();
      paintOutline(g);
      paintMessageBar(g);
    }

    private void setRatio() {
      m_rectWidth  = (this.getWidth()  - 2*HORIZONTAL_MARGIN) / 4;
      m_rectHeight = (this.getHeight() - 2*VERTICAL_MARGIN)   / 10;
    }

    private void paintOutline(Graphics g) {
      g.setColor(Color.gray);
      for(int i=0; i<4; ++i){
        for(int j=0; j<9; ++j)
          g.drawRect(HORIZONTAL_MARGIN+i*m_rectWidth, VERTICAL_MARGIN+j*m_rectHeight, m_rectWidth, m_rectHeight);
      }
    }

    private void paintMessageBar(Graphics g) {
    }

    public void setPaintedId(int id) { m_id = id; }
    public int  getPaintedId()       { return m_id; }
  }

}