package debugger.view;

import java.util.*;
import java.awt.BorderLayout;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import debugger.*;
import debugger.object.*;

public class StatusViewer extends JFrame implements Constants{
  private static final int PATERN_ALL = 0xffff;
  private static final int PATERN_BUILDING = 0x0001;
  private static final int PATERN_ROAD     = 0x0004;
  private static final int PATERN_NODE     = 0x0010;
  private static final int PATERN_CIVILIAN = 0x0100;
  private static final int PATERN_AMBULANCE_TEAM = 0x0200;
  private static final int PATERN_FIRE_BRIGADE = 0x0400;
  private static final int PATERN_POLICE_FORCE = 0x0800;
  private static final int PATERN_HUMANOID = 0x0f00;
  private final DefaultTableModel m_model = new DefaultTableModel();
  private final JTable m_mainTable = new JTable(m_model);

  final JCheckBox chCivilian = new JCheckBox("Civilian", true);
  final JCheckBox chAmbulanceTeam =  new JCheckBox("AmbulanceTeam", true);
  final JCheckBox chFireBrigade = new JCheckBox("FireBrigade", false);
  final JCheckBox chPoliceForce = new JCheckBox("PoliceForce", false);
  final JCheckBox chBuilding    = new JCheckBox("Building", false);
  final JCheckBox chRoad        = new JCheckBox("Road", false);
  private boolean m_changeIndice = true;
  private int m_state = PATERN_CIVILIAN | PATERN_AMBULANCE_TEAM;
  public final List m_selectedIdList = new ArrayList();

  private int m_prevColumn = -1;
  public StatusViewer() {
    super("Status Viewer");
    JScrollPane mainPane = new JScrollPane();
    setTable();
    mainPane.getViewport().add(m_mainTable);
    this.getContentPane().add(mainPane, BorderLayout.CENTER);
    this.getContentPane().add(createToolbar(), BorderLayout.NORTH);
    pack();
    setBounds(100, 600, 600, 200);
  }
  private void setTable() {
    m_mainTable.getSelectionModel().addListSelectionListener(new ListSelectionListener(){
      public void valueChanged(ListSelectionEvent e) {
        mainTableSelected(e);
      }
    });
    final SortDecorator decorator = new SortDecorator(m_mainTable.getModel());
    m_mainTable.setModel(decorator);
    JTableHeader hdr = (JTableHeader)m_mainTable.getTableHeader();
    hdr.addMouseListener(new MouseAdapter(){
      public void mouseClicked(MouseEvent e) {
        TableColumnModel tcm = m_mainTable.getColumnModel();
        int vc = tcm.getColumnIndexAtX(e.getX());
        int mc = m_mainTable.convertColumnIndexToModel(vc);
        boolean smaller = m_prevColumn==mc;
        if(smaller) m_prevColumn = -1;
        else m_prevColumn = mc;
        decorator.sort(vc, smaller);
      }
    });
  }

  private void setStatus(boolean b, int patern) {
    if(b) m_state |= patern;
    else  m_state &= ~patern;
  }
  private void setCheckBoxstatus(int patern) {
    if((m_state & PATERN_HUMANOID) == 0){
      chAmbulanceTeam.setSelected(false);
      chFireBrigade.setSelected(false);
      chPoliceForce.setSelected(false);
      chCivilian.setSelected(false);
    }
    if((m_state & PATERN_BUILDING) == 0){
      chBuilding.setSelected(false);
    }
    if((m_state & PATERN_ROAD) == 0){
      chRoad.setSelected(false);
    }
  }

  private JToolBar createToolbar() {
    JToolBar root = new JToolBar();
    /// listener
    ActionListener listener = new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        Object o = e.getSource();
        boolean hflag=true, bflag=false, rflag=false;
        if(o == chCivilian) setStatus(chCivilian.isSelected(), PATERN_CIVILIAN);
        else if(o == chAmbulanceTeam) setStatus(chAmbulanceTeam.isSelected(), PATERN_AMBULANCE_TEAM);
        else if(o == chFireBrigade) setStatus(chFireBrigade.isSelected(), PATERN_FIRE_BRIGADE);
        else if(o == chPoliceForce) setStatus(chPoliceForce.isSelected(), PATERN_POLICE_FORCE);
        else if(o == chBuilding) {
            bflag = chBuilding.isSelected();
            hflag = false;
            setStatus(bflag, PATERN_BUILDING);
        } else if(o == chRoad) {
          rflag = chRoad.isSelected();
          hflag = false;
          setStatus(rflag, PATERN_ROAD);
        }
        if(!hflag) setStatus(false, PATERN_HUMANOID);
        if(!bflag) setStatus(false, PATERN_BUILDING);
        if(!rflag) setStatus(false, PATERN_ROAD);
        m_changeIndice = true;
        setCheckBoxstatus(m_state);
        setData();
      }
    };
    chCivilian.addActionListener(listener);
    chAmbulanceTeam.addActionListener(listener);
    chFireBrigade.addActionListener(listener);
    chPoliceForce.addActionListener(listener);
    chBuilding.addActionListener(listener);
    chRoad.addActionListener(listener);
    ////////////
    root.add(chCivilian);  root.add(chAmbulanceTeam);
    root.add(chFireBrigade);  root.add(chPoliceForce);
    root.add(chBuilding);  root.add(chRoad);
    return root;
  }

  public void progress() {
    setData();
  }

  private void setData() {
    synchronized(m_model){
      m_model.setRowCount(0);
      setTableIndex(m_state);
    }
  }
  private void addTableData(Object[] data) { m_model.addRow(data); }
  private void addTableData(Vector   data) { m_model.addRow(data); }
  private List getData(List objList) {
    List list = new ArrayList(WORLD.movingObjectList.size());
    Iterator iter = objList.iterator();
    while (iter.hasNext()) {
      Object item = iter.next();
      if(item instanceof Humanoid){
        Humanoid h = (Humanoid)item;
        String[] d = { ""+h.id, ""+h.hp(), ""+h.damage(), ""+h.buriedness(),
                       ""+h.position().id, ""+h.positionExtra()};
        list.add(d);
      } else if(item instanceof Building){
        Building b = (Building)item;
        String[] d = { ""+b.id, ""+b.fieryness(), ""+b.brokenness(),
                       ""+b.x(), ""+b.y()};
        list.add(d);
      } else if(item instanceof Road){
        Road r = (Road)item;
        String[] d = { ""+r.id, ""+r.block(), ""+r.blockedLines(),
                       ""+r.aliveLinesToHead()+" / "+r.aliveLinesToTail(),
                       ""+r.length(), ""+r.head().id, ""+r.tail().id};
        list.add(d);
      }
    }

    return list;
  }
  private void setTableIndex(int patern) {
    String data[];
    List indiceList = new ArrayList();
    List dataList   = new ArrayList();
    Vector vector   = new Vector(7);
    vector.add("ID");
    if((patern & PATERN_HUMANOID) != 0) {
      String[] indice = {"HP", "Damage", "Buriedness", "Position", "PositionEx"};
      indiceList.add(indice);
      if((patern & PATERN_CIVILIAN) != 0) dataList.add(getData(WORLD.civilianList));
      if((patern & PATERN_AMBULANCE_TEAM) != 0) dataList.add(getData(WORLD.ambulanceTeamList));
      if((patern & PATERN_FIRE_BRIGADE) != 0) dataList.add(getData(WORLD.fireBrigadeList));
      if((patern & PATERN_POLICE_FORCE) != 0) dataList.add(getData(WORLD.policeForceList));
    }
    else if((patern & PATERN_BUILDING) != 0){
      String[] indice = {"Feiryness", "Brokenness", "x", "y"};
      indiceList.add(indice);
      dataList.add(getData(WORLD.buildingList));
    }
    else if((patern & PATERN_ROAD) != 0){
      String[] indice = {"Block", "BlockedLine", "Available(H/T)", "Length", "Head", "Tail" };
      indiceList.add(indice);
      dataList.add(getData(WORLD.roadList));
    }
    Iterator iter = indiceList.iterator();
    while (iter.hasNext()) {
      String[] item = (String[])iter.next();
      for (int i = 0; i < item.length; i++) {
        vector.add(item[i]);
      }
    }
    //// make indice
    if(m_changeIndice) {
      m_changeIndice = false;
      makeIndice(vector);
    }
    //// set data
    iter = dataList.iterator();
    while (iter.hasNext()) {
      List d = (List)iter.next();
      Iterator iter2 = d.iterator();
      while (iter2.hasNext()) {
        addTableData((String[])iter2.next());
      }
    }
  }
  private void makeIndice(Vector indice) {
    m_model.setColumnIdentifiers(indice);
  }

  //// selected
  private void mainTableSelected(ListSelectionEvent e) {
    m_selectedIdList.clear();
    int selc = m_mainTable.getSelectedColumn(),
        selr = m_mainTable.getSelectedRow();
    int[] selRows = m_mainTable.getSelectedRows();
    for (int i = 0; i < selRows.length; i++) {
      String data = ""+m_mainTable.getModel().getValueAt(selRows[i], 0);
      m_selectedIdList.add(new Integer(data));
    }
    VIEWER.repaint();
  }
  ///////////////////////////////////////////////////////////////////////////


  ///////////////////////////////////////
  public static void main(String[] args) {
    StatusViewer statusViewer1 = new StatusViewer();
    statusViewer1.addWindowListener(new WindowAdapter(){
      public void windowClosing(WindowEvent e){System.exit(0);}
    });

    statusViewer1.show();
  }

}